#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import numpy as np
from matplotlib import pyplot as p
from matplotlib import colors
import pickle
import os
import json

# Own modules
from models import CBL
from ENDA import experiment
from graphics import *

def load_or_run(settings):

    try:
        exp = pickle.load(open(settings['path']+'/'+settings['filename'], "rb"))
        print('loaded experiment %s'%(settings['path']+'/'+settings['filename']))
    except:
        print('loading %s failed, running it'%(settings['path']+'/'+settings['filename']))
        exp = experiment(settings)
        pickle.dump(exp, open(settings['path']+'/'+settings['filename'], 'wb')) 

    return exp

if __name__ == '__main__':

    workdir = "./runs/seed=181612_enssize=200_EAKF_6hrs/"

    ##################################################################################
    # Check if working directory exists, if not, create it.
    if not os.path.exists(workdir):
        os.makedirs(workdir)

    ##################################################################################
    # Check if the necessary config files exist in the working directory.
    # If not, copy them.
    # for src in args[:-1]:
    #   os.symlink(src, os.path.join(dst, os.path.dirname(src)))

    ########################################################################
    # Manuscript figures
    fig01 = False # CHECKED OK # Sensitivity to p
    fig02 = False # CHECKED OK # Successful PE experiment
    fig03 = False # CHECKED OK # Consistency checks
    fig04 = False # CHECKED OK # Experiments with varying weights
    fig05 = False # CHECKED OK # Experiments with systematic model errors
    fig06 = False # CHECKED OK # Analysis increments in model error experiments
    fig07 = False # CHECKED OK # Experiment with error growth
    fig08 = False # CHECKED OK # Comparison successful vs unsuccessful PE
    fig09 = False  # CHECKED OK # Comparison EAKF vs LETKF
    fig10 = True  # CHECKED OK # DA diagnostics (profiles) for perfect-obs experiments
    
    # Other figures
    opt01 = False # CHECKED OK # assimilation of a single observation
    opt02 = False # CHECKED OK # assimilation of profiles at two times
    opt03 = False # CHECKED OK # assimilation of real observations
    opt04 = False # CHECKED OK # assimilation of real observations, different spinup times
    opt05 = False # CHECKED OK # Check impact of localization cutoff
    opt06 = False # CHECKED OK # Check impact of RTPS variance inflation factor for parameters
    opt07 = False # CHECKED OK # Check impact of RTPS variance inflation factor for state vector
    opt08 = False # CHECKED OK # Check sensitivity to observation sorting
    opt09 = False # CHECKED OK # Plot covariance matrix
    opt10 = False # CHECKED OK # Consistency checks
    opt11 = False # CHECKED OK # An experiment without any variance inflation
    opt12 = False # CHECKED OK # Old version of Figure 08

    # A test to check if results are exactly reproducible
    check_reproducibility = False

    if fig01:

        # Create a copy of the default settings
        # and update as necessary
        with open(workdir+'/default_cbl.json', 'r') as fp:
            cbl_settings = json.load(fp)
        maxtime = 21600
        cbl_settings['maxtime'] = maxtime

        # Panel a) deterministic run using default settings
        cbl_det = CBL(cbl_settings)
        cbl_det.initialize(1)
        cbl_det.run(output_full_history=True)

        # Panel b) sensitivity to p; use a smaller domain and higher
        # resolution for better-looking plots.
        zmax = 2000
        p_factors = [0.5,1.5,4.5]
        cbl_settings['dz'] = 25
        cbl_settings['ztop'] = zmax

        theta_profiles = []
        for pfac in p_factors:
            cbl_settings['pfac'] = pfac
            cbl_pf = CBL(cbl_settings)
            cbl_pf.initialize(1)
            cbl_pf.run()
            theta_profiles.append(cbl_pf.x[:cbl_pf.nz])

        # Panel c) spread induced by p
        # Do a free ensemble run (ensemble size set expliclity)
        # Set size to 200 members
        with open(workdir+'/default_cbl.json', 'r') as fp:
            cbl_settings_free = json.load(fp)
        cbl_settings_free['maxtime'] = maxtime
        cbl_settings_free['perturb_ensemble_state'] = False
        cbl_free = CBL(cbl_settings_free)
        cbl_free.initialize(200)
        cbl_free.run(output_full_history=True)

        # Make plots
        ncont = 13
        fig, [[ax4, ax2],[ax1, ax3], [ax5, ax6] ] = p.subplots(3,2,constrained_layout=True)
        fig.set_size_inches(6,9)

        c1 = ax1.pcolormesh(cbl_det.history['time']/3600,cbl_det.zt,cbl_det.history['theta'],vmin=290,vmax=296)
        ax1.set_ylim([0,zmax])
        ax1.set_ylabel(r'Height (m)')
        ax1.set_xlabel(r'Time (h)')
        ax1.set_xticks(np.arange((maxtime+1)/3600))
        ax1.set_title(r'c) $\overline{\theta}$ (K)')
        p.colorbar(c1,orientation='horizontal')
        ax1.contour(cbl_det.history['time']/3600,cbl_det.zt,cbl_det.history['theta'],
                    np.linspace(cbl_det.theta_0,cbl_det.theta_0+cbl_det.gamma*zmax,ncont),
                    colors='white',
                    linestyles='--',
                    linewidths=0.75)

        ax2 = plot_p(p_factors,theta_profiles,cbl_pf.zt,None,ax=ax2)
        ax2.set_ylabel(r'Height (m)')
        ax2.set_xlabel(r'$\overline{\theta}$ (K)')
        ax2.set_xlim([291,297])
        ax2.set_ylim([0,zmax])
        ax2.legend(loc=4,frameon=False)
        ax2.set_title(r'b)  $\overline{\theta}$ sensitivity to $p$')

        ax3,c3 = plot_free_ensemble(cbl_free,show='spread',ax=ax3)
        ax3.set_ylabel(r'Height (m)')
        ax3.set_title(r'd) $\sigma_\theta$ (K)')
        ax3.set_xlabel('Time (h)')
        ax3.set_xticks(np.arange((maxtime+1)/3600))
        p.colorbar(c3,orientation='horizontal')

        zoverh = np.linspace(0,1,101)
        for pfac in p_factors:
            Koverkws = zoverh*(1-zoverh)**pfac
            ax4.plot(Koverkws,zoverh,label='$p=%4.1f$'%pfac)
        ax4.set_title(r'a) $K_h$ sensitivity to $p$')
        ax4.set_xlabel('$K_h/(\kappa w_s h)$')
        ax4.set_ylabel('$z/h$')
        ax4.set_xlim([0,0.5])
        ax4.legend(loc=4,frameon=False)

        ax5,c5 = plot_free_ensemble(cbl_free,show='corr',ax=ax5,cmap='RdBu_r')
        ax5.set_ylabel(r'Height (m)')
        ax5.set_title(r'e) $\rho(p,\theta\prime)$')
        ax5.set_xlabel('Time (h)')
        ax5.set_xticks(np.arange((maxtime+1)/3600))
        p.colorbar(c5,orientation='horizontal')

        ax6 = plot_linearity(cbl_free,ax=ax6)
        ax6.set_ylabel(r'$\overline{\theta}$ (K)')
        ax6.set_title(r'f) $p$ vs $\overline{\theta}$, $t=6$ h')
        ax6.set_xlabel(r'$p$')

        #p.setp(ax2.get_yticklabels(), visible=False)
        #p.setp(ax3.get_yticklabels(), visible=False)
        fig.savefig('fig01.png',format='png',dpi=300)
        p.close(fig)

    if fig02:

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_A1 = json.load(fp)
        exp_A1 = load_or_run(da_settings_A1)

        plot_overview(exp_A1,"A$_1$",'fig02.png')

    if fig03:

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_A1 = json.load(fp)
        exp_A1 = load_or_run(da_settings_A1)

        with open(workdir+'/da_exp_A2.json', 'r') as fp:
            da_settings_A2 = json.load(fp)
        exp_A2 = load_or_run(da_settings_A2)

        with open(workdir+'/da_exp_B1.json', 'r') as fp:
            da_settings_B1 = json.load(fp)
        exp_B1 = load_or_run(da_settings_B1)

        with open(workdir+'/da_exp_B2.json', 'r') as fp:
            da_settings_B2 = json.load(fp)
        exp_B2 = load_or_run(da_settings_B2)

        with open(workdir+'/da_exp_B3.json', 'r') as fp:
            da_settings_B3 = json.load(fp)
        exp_B3 = load_or_run(da_settings_B3)

        with open(workdir+'/da_exp_B4.json', 'r') as fp:
            da_settings_B4 = json.load(fp)
        exp_B4 = load_or_run(da_settings_B4)

        with open(workdir+'/da_exp_B4_noPE.json', 'r') as fp:
            da_settings_B4_noPE = json.load(fp)
        exp_B4_noPE = load_or_run(da_settings_B4_noPE)

        with open(workdir+'/da_exp_C1.json', 'r') as fp:
            da_settings_C1 = json.load(fp)
        exp_C1 = load_or_run(da_settings_C1)

        with open(workdir+'/da_exp_C2.json', 'r') as fp:
            da_settings_C2 = json.load(fp)
        exp_C2 = load_or_run(da_settings_C2)

        with open(workdir+'/da_exp_D1.json', 'r') as fp:
            da_settings_D1 = json.load(fp)
        exp_D1 = load_or_run(da_settings_D1)

        with open(workdir+'/da_exp_D2.json', 'r') as fp:
            da_settings_D2 = json.load(fp)
        exp_D2 = load_or_run(da_settings_D2)

        # Plots: Scatter plots of Desroziers statistics
        #experiments1 = [exp_A1, exp_B1, exp_B2, exp_B3, exp_B4, exp_C1, exp_C2, exp_D1, exp_D2 ]
        #labels1 = [r'A$_1$', r'B$_1$', r'B$_2$', r'B$_3$', r'B$_4$', r'C$_1$', r'C$_2$', r'D$_1$', r'D$_2$' ]
        #experiments2 = [exp_B2, exp_B3, exp_D1, exp_D2 ]
        #labels2      = [r'B$_2$', r'B$_3$', r'D$_1$', r'D$_2$' ]
        #experiments1 = [exp_A1, exp_A2, exp_B1, exp_B4, exp_C1, exp_C2 ]
        #labels1      = [r'A$_1$', r'A$_2$',r'B$_1$', r'B$_4$', r'C$_1$', r'C$_2$' ]
        experiments1 = [exp_A1, exp_B1, exp_B4, exp_C1, exp_C2 ]
        labels1      = [r'A$_1$', r'B$_1$', r'B$_4$', r'C$_1$', r'C$_2$' ]
        experiments2 = [exp_B2, exp_B3, exp_D2 ]
        labels2      = [r'B$_2$', r'B$_3$', r'D' ]

        fig = p.figure(0)
        fig.set_size_inches(6,3)
        ax1 = fig.add_subplot(1,2,1)
        ax2 = fig.add_subplot(1,2,2)

        ax1 = plot_desroziers([i.dg for i in experiments1],labels=labels1,ax=ax1)
        ax2 = plot_desroziers([i.dg for i in experiments2],labels=labels2,ax=ax2)
        ax1.set_title(r"a) Exp. with $\sigma_{y_o}^\mathsf{assim}=0.1$ K")
        ax2.set_title(r"b) Exp. with $\sigma_{y_o}^\mathsf{assim}=1$ K")

        fig.tight_layout()
        fig.savefig('fig03.png',format='png',dpi=150)
        p.close(fig)

    if fig04:

        with open(workdir+'/da_exp_B1.json', 'r') as fp:
            da_settings_B1 = json.load(fp)
        exp_B1 = load_or_run(da_settings_B1)

        with open(workdir+'/da_exp_B2.json', 'r') as fp:
            da_settings_B2 = json.load(fp)
        exp_B2 = load_or_run(da_settings_B2)

        with open(workdir+'/da_exp_B3.json', 'r') as fp:
            da_settings_B3 = json.load(fp)
        exp_B3 = load_or_run(da_settings_B3)

        with open(workdir+'/da_exp_B4.json', 'r') as fp:
            da_settings_B4 = json.load(fp)
        exp_B4 = load_or_run(da_settings_B4)

        fig, [[ax1, ax2], [ax3, ax4]] = p.subplots(2,2,constrained_layout=True)
        fig.set_size_inches(6,4)
        #
        ax1 = plot_CBL_PE(exp_B1,None,ax=ax1,show_true_parameter=True)
        ax1.set_title(r'a) Exp. B$_1$')
        ax1.set_xlabel('Time (h)')
        ax1.set_ylabel(r'$p$')
        #
        ax2 = plot_CBL_PE(exp_B2,None,ax=ax2,show_true_parameter=True)
        ax2.set_title(r'b) Exp. B$_2$')
        ax2.set_xlabel('Time (h)')
        ax2.set_ylabel(r'$p$')
        #
        ax3 = plot_CBL_PE(exp_B3,None,ax=ax3,show_true_parameter=True)
        ax3.set_title(r'c) Exp. B$_3$')
        ax3.set_xlabel('Time (h)')
        ax3.set_ylabel(r'$p$')
        #
        ax4 = plot_CBL_PE(exp_B4,None,ax=ax4,show_true_parameter=True)
        ax4.set_title(r'd) Exp. B$_4$')
        ax4.set_xlabel('Time (h)')
        ax4.set_ylabel(r'$p$')
        #
        ax1.set_yticks([0,1,2,3,4,5])
        ax2.sharey(ax1)
        ax3.set_yticks([0,1,2,3,4,5])
        ax4.sharey(ax3)
        #
        fig.savefig('fig04.png',format='png',dpi=300)
        p.close(fig)

    if fig05:

        with open(workdir+'/da_exp_C1.json', 'r') as fp:
            da_settings_C1 = json.load(fp)
        exp_C1 = load_or_run(da_settings_C1)

        with open(workdir+'/da_exp_C2.json', 'r') as fp:
            da_settings_C2 = json.load(fp)
        exp_C2 = load_or_run(da_settings_C2)

        fig, [ax1, ax2] = p.subplots(1,2,constrained_layout=True)
        fig.set_size_inches(6,2)
        #
        ax1 = plot_CBL_PE(exp_C1,None,ax=ax1,show_true_parameter=True)
        ax1.set_title(r'a) Exp. C$_1$')
        ax1.set_xlabel('Time (h)')
        ax1.set_ylabel(r'$p$')
        #
        ax2 = plot_CBL_PE(exp_C2,None,ax=ax2,show_true_parameter=True)
        ax2.set_title(r'b) Exp. C$_2$')
        ax2.set_xlabel('Time (h)')
        ax2.set_ylabel(r'$p$')
        #
        ax1.set_yticks([0,1,2,3,4,5])
        ax2.sharey(ax1)
        #
        fig.savefig('fig05.png',format='png',dpi=300)
        p.close(fig)

    if fig06:

        with open(workdir+'/da_exp_C1.json', 'r') as fp:
            da_settings_C1 = json.load(fp)
        exp_C1 = load_or_run(da_settings_C1)

        with open(workdir+'/da_exp_C2.json', 'r') as fp:
            da_settings_C2 = json.load(fp)
        exp_C2 = load_or_run(da_settings_C2)

        npar2 = exp_C1.nr.parameter_number
        npar3 = exp_C2.nr.parameter_number
        updates_C1 = exp_C1.da.analyses[:,:-npar2,:].mean(axis=2).T - exp_C1.da.backgrounds[:,:-npar2,:].mean(axis=2).T
        updates_C2 = exp_C2.da.analyses[:,:-npar3,:].mean(axis=2).T - exp_C2.da.backgrounds[:,:-npar3,:].mean(axis=2).T

        fig, [ax1, ax2] = p.subplots(1,2,constrained_layout=True)
        fig.set_size_inches(6,3)
        c1 = ax1.pcolormesh(exp_C1.da.time/3600,exp_C1.nr.zt,updates_C1,vmin=-0.03,vmax=0.03,cmap='RdBu_r')
        c2 = ax2.pcolormesh(exp_C2.da.time/3600,exp_C2.nr.zt,updates_C2,vmin=-0.03,vmax=0.03,cmap='RdBu_r')
        #
        ax1.set_ylim([0,2000])
        ax2.set_ylim([0,2000])
        ax1.set_title(r'a) Exp. C$_1$, $\overline{\theta}_a-\overline{\theta}_b$')
        ax2.set_title(r'b) Exp. C$_2$, $\overline{\theta}_a-\overline{\theta}_b$')
        ax1.set_xlabel('Time (h)')
        ax2.set_xlabel('Time (h)')
        ax1.set_ylabel('Height (m)')
        ax2.sharey(ax1)
        maxtime = exp_C1.trun
        ax1.set_xlim([0,maxtime/3600])
        ax2.set_xlim([0,maxtime/3600])
        ax1.set_xticks(np.arange((maxtime+1)/3600))
        ax2.set_xticks(np.arange((maxtime+1)/3600))
        p.setp(ax2.get_yticklabels(), visible=False)
        #
        p.colorbar(c1,orientation='horizontal')
        p.colorbar(c2,orientation='horizontal')
        #
        fig.savefig('fig06.png',format='png',dpi=300)
        p.close(fig)

    if fig07:

        with open(workdir+'/da_exp_D2.json', 'r') as fp:
            da_settings_D2 = json.load(fp)
        exp_D = load_or_run(da_settings_D2)

        fig, [ax0, ax1] = p.subplots(1,2,constrained_layout=True)
        fig.set_size_inches(6,3)
        #
        ax0,c0 = plot_CBL_identifiability(exp_D,da_settings_D2['obs_error_sdev_assimilate'],ax=ax0)
        ax0.set_title(r'a) Exp. D, $\rho(p\prime\prime,y_b}$)')
        ax0.set_xlabel('Time (h)')
        ax0.set_ylabel('Height (m)')
        ax1 = plot_CBL_PE(exp_D,None,ax=ax1,show_true_parameter=True)
        ax1.set_title(r'b) Exp. D, evolution of $p$')
        ax1.set_xlabel('Time (h)')
        ax1.set_yticks([0,1,2,3,4,5])
        maxtime = exp_D.trun
        ax0.set_xlim([0,maxtime/3600])
        ax1.set_xlim([0,maxtime/3600])
        ax0.set_xticks(np.arange((maxtime+1)/3600))
        ax1.set_xticks(np.arange((maxtime+1)/3600))
        p.colorbar(c0,orientation='horizontal')
        #
        fig.savefig('fig07.png',format='png',dpi=300)
        p.close(fig)

    if fig08:

        with open(workdir+'/da_exp_B4.json', 'r') as fp:
            da_settings_B4 = json.load(fp)
        exp_B4 = load_or_run(da_settings_B4)

        fig, [ax3,ax2] = p.subplots(1,2,constrained_layout=True)
        fig.set_size_inches(6,3)
        #
        [ax2,ax3],c2,c3 = plot_CBL_identifiability(exp_B4,da_settings_B4['obs_error_sdev_assimilate'],ax=[ax3,ax2])
        ax2.set_title(r'a) Exp. B$_4$, $\delta\overline{y}$ (K)')
        ax2.set_xlabel('Time (h)')
        ax2.set_ylabel('Height (m)')
        ax3.set_title(r'b) Exp. B$_4$, $\delta\overline{y}\cdot(\sigma_{p\prime\prime}/\sigma_{y^b})$')
        ax3.set_xlabel('Time (h)')
        ax3.set_ylabel('Height (m)')
        maxtime = exp_B4.trun
        ax2.set_xlim([0,maxtime/3600])
        ax3.set_xlim([0,maxtime/3600])
        ax2.set_xticks(np.arange((maxtime+1)/3600))
        ax3.set_xticks(np.arange((maxtime+1)/3600))
        p.colorbar(c2,orientation='horizontal')
        p.colorbar(c3,orientation='horizontal')
        #
        fig.savefig('fig08.png',format='png',dpi=300)
        p.close(fig)

    if fig09:

        workdir_eakf = "./runs/seed=181612_enssize=200_EAKF_6hrs/"
        workdir_letkf = "./runs/seed=181612_enssize=200_LETKF_6hrs/"

        with open(workdir_eakf+'/da_exp_A1.json', 'r') as fp:
            da_settings_A1 = json.load(fp)
        exp_A1a = load_or_run(da_settings_A1)

        with open(workdir_eakf+'/da_exp_B1.json', 'r') as fp:
            da_settings_B1 = json.load(fp)
        exp_B1a = load_or_run(da_settings_B1)

        with open(workdir_eakf+'/da_exp_B2.json', 'r') as fp:
            da_settings_B2 = json.load(fp)
        exp_B2a = load_or_run(da_settings_B2)

        with open(workdir_eakf+'/da_exp_B3.json', 'r') as fp:
            da_settings_B3 = json.load(fp)
        exp_B3a = load_or_run(da_settings_B3)

        with open(workdir_eakf+'/da_exp_B4.json', 'r') as fp:
            da_settings_B4 = json.load(fp)
        exp_B4a = load_or_run(da_settings_B4)

        with open(workdir_eakf+'/da_exp_C1.json', 'r') as fp:
            da_settings_C1 = json.load(fp)
        exp_C1a = load_or_run(da_settings_C1)

        with open(workdir_eakf+'/da_exp_C2.json', 'r') as fp:
            da_settings_C2 = json.load(fp)
        exp_C2a = load_or_run(da_settings_C2)

        with open(workdir_eakf+'/da_exp_D2.json', 'r') as fp:
            da_settings_D2 = json.load(fp)
        exp_D2a = load_or_run(da_settings_D2)

        with open(workdir_letkf+'/da_exp_A1.json', 'r') as fp:
            da_settings_A1 = json.load(fp)
        exp_A1b = load_or_run(da_settings_A1)

        with open(workdir_letkf+'/da_exp_B1.json', 'r') as fp:
            da_settings_B1 = json.load(fp)
        exp_B1b = load_or_run(da_settings_B1)

        with open(workdir_letkf+'/da_exp_B2.json', 'r') as fp:
            da_settings_B2 = json.load(fp)
        exp_B2b = load_or_run(da_settings_B2)

        with open(workdir_letkf+'/da_exp_B3.json', 'r') as fp:
            da_settings_B3 = json.load(fp)
        exp_B3b = load_or_run(da_settings_B3)

        with open(workdir_letkf+'/da_exp_B4.json', 'r') as fp:
            da_settings_B4 = json.load(fp)
        exp_B4b = load_or_run(da_settings_B4)

        with open(workdir_letkf+'/da_exp_C1.json', 'r') as fp:
            da_settings_C1 = json.load(fp)
        exp_C1b = load_or_run(da_settings_C1)

        with open(workdir_letkf+'/da_exp_C2.json', 'r') as fp:
            da_settings_C2 = json.load(fp)
        exp_C2b = load_or_run(da_settings_C2)

        with open(workdir_letkf+'/da_exp_D2.json', 'r') as fp:
            da_settings_D2 = json.load(fp)
        exp_D2b = load_or_run(da_settings_D2)

        """
        exp_Aa = pickle.load(open(path_EAKF+"exp_A1.pickle", "rb"))
        exp_B1a = pickle.load(open(path_EAKF+"exp_B1.pickle", "rb"))
        exp_B2a = pickle.load(open(path_EAKF+"exp_B2.pickle", "rb"))
        exp_B3a = pickle.load(open(path_EAKF+"exp_B3.pickle", "rb"))
        exp_B4a = pickle.load(open(path_EAKF+"exp_B4.pickle", "rb"))
        exp_C1a = pickle.load(open(path_EAKF+"exp_C1.pickle", "rb"))
        exp_C2a = pickle.load(open(path_EAKF+"exp_C2.pickle", "rb"))
        exp_D2a = pickle.load(open(path_EAKF+"exp_D2.pickle", "rb"))

        exp_Ab = pickle.load(open(path_LETKF+"exp_A1.pickle", "rb"))
        exp_B1b = pickle.load(open(path_LETKF+"exp_B1.pickle", "rb"))
        exp_B2b= pickle.load(open(path_LETKF+"exp_B2.pickle", "rb"))
        exp_B3b = pickle.load(open(path_LETKF+"exp_B3.pickle", "rb"))
        exp_B4b = pickle.load(open(path_LETKF+"exp_B4.pickle", "rb"))
        exp_C1b = pickle.load(open(path_LETKF+"exp_C1.pickle", "rb"))
        exp_C2b = pickle.load(open(path_LETKF+"exp_C2.pickle", "rb"))
        exp_D2b = pickle.load(open(path_LETKF+"exp_D2.pickle", "rb"))
        """

        fig, [ax1, ax2] = p.subplots(1,2)
        fig.set_size_inches(6,3)
        #
        linecolors = ['darkviolet','chocolate','darkorange','orange','gold','darkgreen','seagreen','royalblue','black']
        labels = [r'Exp. A$_1$',r'Exp. B$_1$',r'Exp. B$_2$',r'Exp. B$_3$',r'Exp. B$_4$',r'Exp. C$_1$',r'Exp. C$_2$',r'Exp. D']
        experiments1 = [exp_A1a,exp_B1a,exp_B2a,exp_B3a,exp_B4a,exp_C1a,exp_C2a,exp_D2a]
        experiments2 = [exp_A1b,exp_B1b,exp_B2b,exp_B3b,exp_B4b,exp_C1b,exp_C2b,exp_D2b]
        #
        for exp,label,linecolor in zip(experiments1,labels,linecolors):
            # Experiment settings
            true_value = exp.nr.pfac
            parameter_number = exp.nr.parameter_number
            parameter_id = 0
            pt = exp.nr.parameter_transform
            par_tran = exp.da.backgrounds[:,-parameter_number:,:][:,parameter_id,:]
            par_phys = pt[parameter_id](par_tran,kind='inv')
            initpar_phys = pt[parameter_id](exp.da.initial_perturbed_parameters,kind='inv')
            t = exp.da.time/3600.

            # Initial parameter distribution
            ax1.step([0,t[0]],np.mean(initpar_phys,axis=1)*np.array([1,1]),color=linecolor,label=label)

            # Later times
            ax1.step(t,np.mean(par_phys,axis=1),color=linecolor)

        for exp,label,linecolor in zip(experiments2,labels,linecolors):
            # Experiment settings
            true_value = exp.nr.pfac
            parameter_number = exp.nr.parameter_number
            parameter_id = 0
            pt = exp.nr.parameter_transform
            par_tran = exp.da.backgrounds[:,-parameter_number:,:][:,parameter_id,:]
            par_phys = pt[parameter_id](par_tran,kind='inv')
            initpar_phys = pt[parameter_id](exp.da.initial_perturbed_parameters,kind='inv')
            t = exp.da.time/3600.

            # Initial parameter distribution
            ax2.step([0,t[0]],np.mean(initpar_phys,axis=1)*np.array([1,1]),color=linecolor)

            # Later times
            ax2.step(t,np.mean(par_phys,axis=1),color=linecolor)

        # Other stuff
        ax1.axhline(y=true_value,linestyle='--',color='black')
        ax1.set_title(r'a) EAKF experiments')
        ax1.set_xlim([0,t.max()])
        ax1.set_xlabel('Time (h)')
        ax1.set_ylabel(r'$p$')
        ax1.set_yticks([1,2,3])
        #
        ax2.axhline(y=true_value,linestyle='--',color='black')
        ax2.set_title(r'b) LETKF experiments')
        ax2.set_xlim([0,t.max()])
        ax2.set_xlabel('Time (h)')
        ax2.set_ylabel(r'$p$')
        ax2.set_yticks([1,2,3])
        #
        ax2.sharey(ax1)
        fig.legend(loc=8,ncol=4,frameon=False)
        fig.subplots_adjust(bottom=0.35, wspace=0.38)
        #
        fig.savefig('fig09.png',format='png',dpi=300)
        p.close(fig)

    if fig10:

        # For the answer to reviewer A
        """
        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_A1 = json.load(fp)
        exp1 = load_or_run(da_settings_A1)
        label = "A$_1$"

        with open(workdir+'/da_exp_A2.json', 'r') as fp:
            da_settings_A2 = json.load(fp)
        exp2 = load_or_run(da_settings_A2)
        """

        # For the revised manuscript
        with open(workdir+'/da_exp_B4.json', 'r') as fp:
            da_settings_B4 = json.load(fp)
        exp1 = load_or_run(da_settings_B4)
        label = "B$_4$"

        with open(workdir+'/da_exp_B4_noPE.json', 'r') as fp:
            da_settings_B4_noPE = json.load(fp)
        exp2 = load_or_run(da_settings_B4_noPE)
        
        # Diagnostics wrt truths
        fig, [[ax1, ax2], [ax3, ax4]] = p.subplots(2,2,constrained_layout=True)
        fig.set_size_inches(6,4)
        plot_diagnostics(exp1,show='errors',axes=[ax1,ax2,ax3,ax4],label='B$_4$ with PE',linecolor='#1f77b4',zmax=1500)
        plot_diagnostics(exp2,show='errors',axes=[ax1,ax2,ax3,ax4],label='B$_4$ without PE',linecolor='#9EC9E9',zmax=1500)
        ax1.set_title('a) Analysis error')
        ax1.set_xlabel(r'RMSE$_{\overline{y}_a}$')
        ax2.set_title('b) First-guess error')
        ax2.set_xlabel(r'RMSE$_{\overline{y}_b}$')
        ax3.set_title('c) Error reduction')
        ax3.set_xlabel(r'RMSE$_{\overline{y}_b}$-RMSE$_{\overline{y}_a}$')
        ax4.set_title('d) Consistency of innovations')
        ax4.set_xlabel(r'$(\sigma_{y_b}+\sigma_{y_o})$/RMSE$_{\overline{y}_b}$')
        ax1.set_ylabel('height (m)')
        ax3.set_ylabel('height (m)')
        ax1.legend(frameon=False,fontsize=8,loc=7)
        #ax1.set_xlim([0,0.2])
        #ax2.set_xlim([0,0.2])
        #ax3.set_xlim([0,0.02])
        #ax4.set_xlim([0,10])
        fig.savefig('fig10.png',format='png',dpi=300)
        p.close(fig)

        # Diagnostics wrt truths
        fig, [[ax1, ax2], [ax3, ax4]] = p.subplots(2,2,constrained_layout=True)
        fig.set_size_inches(6,4)
        plot_diagnostics(exp1,show='errors',axes=[ax1,ax2,ax3,ax4],label='with PE',linecolor='#1f77b4')
        plot_diagnostics(exp2,show='errors',axes=[ax1,ax2,ax3,ax4],label='without PE',linecolor='#9EC9E9')
        ax1.set_title('a) Analysis error')
        ax1.set_xlabel(r'RMSE$^a_\theta$')
        ax2.set_title('b) First-guess error')
        ax2.set_xlabel(r'RMSE$^b_\theta$')
        ax3.set_title('c) Error reduction')
        ax3.set_xlabel(r'RMSE$^b_\theta-$RMSE$^a_\theta$')
        ax4.set_title('d) Spread-error consistency')
        ax4.set_xlabel(r'$\sigma^b_\theta$/RMSE$^b_\theta$')
        ax1.set_ylabel('height (m)')
        ax3.set_ylabel('height (m)')
        #ax1.set_xlim([0,0.2])
        #ax2.set_xlim([0,0.2])
        #ax3.set_xlim([0,0.02])
        #ax4.set_xlim([0,10])
        fig.savefig('diagnostics_profiles_with_errors.png',format='png',dpi=300)
        p.close(fig)

        # Diagnostics wrt observations
        fig, [[ax1, ax2], [ax3, ax4]] = p.subplots(2,2,constrained_layout=True)
        fig.set_size_inches(6,4)
        plot_diagnostics(exp1,show='deviations',axes=[ax1,ax2,ax3,ax4],label='with PE',linecolor=u'#1f77b4')
        plot_diagnostics(exp2,show='deviations',axes=[ax1,ax2,ax3,ax4],label='without PE',linecolor=u'#9EC9E9')
        ax1.set_title('a) Analysis error')
        ax1.set_xlabel(r'RMSD$^a_\theta$')
        ax2.set_title('b) First-guess error')
        ax2.set_xlabel(r'RMSD$^b_\theta$')
        ax3.set_title('c) Error reduction')
        ax3.set_xlabel(r'RMSD$^b_\theta-$RMSD$^a_\theta$')
        ax4.set_title('d) Spread-error consistency')
        ax4.set_xlabel(r'$\sigma^b_\theta$/RMSD$^b_\theta$')
        ax1.set_ylabel('height (m)')
        ax3.set_ylabel('height (m)')
        #ax1.set_xlim([0,0.2])
        #ax2.set_xlim([0,0.2])
        #ax3.set_xlim([0,0.02])
        #ax4.set_xlim([0,10])
        fig.savefig('diagnostics_profiles_with_deviations.png',format='png',dpi=300)
        p.close(fig)

        # Diagnostics wrt truths and observations compared
        if exp1.da.obs_coordinates.ndim == 1:
            z = np.sort(exp1.da.obs_coordinates)
        elif exp1.da.obs_coordinates.ndim == 2:
            z = np.sort(exp1.da.obs_coordinates[0,:])

        sigma_o = exp1.dg.varo**0.5
        brmsd = exp1.dg.bRMSD_t
        bsprd = exp1.dg.bSprd_t
        brmse = exp1.dg.bRMSE_t

        fig, ax1 = p.subplots(1,1,constrained_layout=True)
        fig.set_size_inches(4,3)
        ax1.plot(np.sqrt(bsprd**2+sigma_o**2)/brmse,z,color=u'#1f77b4',label=r'$\sigma^b_\theta$/RMSE$^b_\theta$; reference is truth')
        ax1.plot(np.sqrt(bsprd**2+sigma_o**2)/brmsd,z,color=u'#9EC9E9',label=r'$\sigma^b_\theta$/RMSD$^b_\theta$; reference is observations')
        ax1.set_title('Exp. %s'%label)
        ax1.set_xlabel(r'spread/error ratio')
        ax1.set_ylabel('height (m)')
        ax1.legend(frameon=False,fontsize=8,loc=7)
        fig.savefig('diagnostics_profiles_errors_vs_deviations.png',format='png',dpi=300)
        p.close(fig)

    if opt01:

        with open('./default_da.json', 'r') as fp:
            da_settings = json.load(fp)
        exp = load_or_run(da_settings)

        plot_CBL_assimilation(exp,'single_assimilation.png')

    if opt02:

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_A1 = json.load(fp)
        exp1 = load_or_run(da_settings_A1)

        plot_profiles(exp_A1,da_settings_A1,'profiles.png')

    if opt03:

        # Control experiment (an OSSE with nature run)
        with open('./runs/real/da_exp_RC.json', 'r') as fp:
            da_settings_RC = json.load(fp)
        exp_RC = load_or_run(da_settings_RC)

        # Test with "real" observations (no nature run)
        with open('./runs/real/da_exp_R.json', 'r') as fp:
            da_settings_R = json.load(fp)
        exp_R = load_or_run(da_settings_R)

        #########################################################################################à
        # Plots
        for exp, label in zip([exp_RC, exp_R],['RC','R']):
            plot_overview(exp,label,'exp_%s.png'%label)

        plot_profiles(exp_RC,da_settings_RC,'exp_RC_profiles.png')
        plot_profiles(exp_R,da_settings_R,'exp_R_profiles.png')

    if opt04:

        # Test with "real" observations (no nature run)
        with open('./runs/real/da_exp_R.json', 'r') as fp:
            da_settings_S0 = json.load(fp)
        exp_S0 = load_or_run(da_settings_S0)
        exp_S0.label = 'S0'

        # Additional tests with longer spinupt time
        with open('./runs/real/da_exp_S1.json', 'r') as fp:
            da_settings_S1 = json.load(fp)
        exp_S1 = load_or_run(da_settings_S1)

        with open('./runs/real/da_exp_S2.json', 'r') as fp:
            da_settings_S2 = json.load(fp)
        exp_S2 = load_or_run(da_settings_S2)

        with open('./runs/real/da_exp_S3.json', 'r') as fp:
            da_settings_S3 = json.load(fp)
        exp_S3 = load_or_run(da_settings_S3)

        for exp,label in zip([exp_S0,exp_S1,exp_S2,exp_S3],['S0','S1','S2','S3']):
            plot_overview(exp,label,'exp_%s.png'%label)

    if opt05:

        with open(workdir+'/da_exp_L1.json', 'r') as fp:
            da_settings_localization_1 = json.load(fp)
        exp_localization_1 = load_or_run(da_settings_localization_1)

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_localization_2 = json.load(fp)
        exp_localization_2 = load_or_run(da_settings_localization_2)
        exp_localization_2.label = 'L2'

        with open(workdir+'/da_exp_L3.json', 'r') as fp:
            da_settings_localization_3 = json.load(fp)
        exp_localization_3 = load_or_run(da_settings_localization_3)

        with open(workdir+'/da_exp_L4.json', 'r') as fp:
            da_settings_localization_4 = json.load(fp)
        exp_localization_4 = load_or_run(da_settings_localization_4)

        for exp,settings,label in zip([exp_localization_1,exp_localization_2,exp_localization_3,exp_localization_4],\
                                      [da_settings_localization_1,da_settings_localization_2,da_settings_localization_3,da_settings_localization_4],
                                      [r"L$_1$",r"L$_2$",r"L$_3$",r"L$_4$"]):
            print(exp.localization_cutoff)

            plot_overview(exp,label,'exp_%s.png'%exp.label)

    if opt06:

        with open(workdir+'/da_exp_I1.json', 'r') as fp:
            da_settings_inflation_1 = json.load(fp)
        exp_inflation_1 = load_or_run(da_settings_inflation_1)

        with open(workdir+'/da_exp_I2.json', 'r') as fp:
            da_settings_inflation_2 = json.load(fp)
        exp_inflation_2 = load_or_run(da_settings_inflation_2)

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_inflation_3 = json.load(fp)
        exp_inflation_3 = load_or_run(da_settings_inflation_3)
        exp_inflation_3.label = 'I3'

        with open(workdir+'/da_exp_I4.json', 'r') as fp:
            da_settings_inflation_4 = json.load(fp)
        exp_inflation_4 = load_or_run(da_settings_inflation_4)

        for exp,settings,label in zip([exp_inflation_1,exp_inflation_2,exp_inflation_3,exp_inflation_4],\
                                      [da_settings_inflation_1,da_settings_inflation_2,da_settings_inflation_3,da_settings_inflation_4],
                                      [r"I$_1$",r"I$_2$",r"I$_3$",r"I$_4$"]):
            print(exp.cbl_settings["parameter_inflation_rtps_alpha"])

            plot_overview(exp,label,'exp_%s.png'%exp.label,show_spread=True,show_transformed_parameter=True)

    if opt07:

        with open(workdir+'/da_exp_I5.json', 'r') as fp:
            da_settings_inflation_5 = json.load(fp)
        exp_inflation_5 = load_or_run(da_settings_inflation_5)

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_inflation_6 = json.load(fp)
        exp_inflation_6 = load_or_run(da_settings_inflation_6)
        exp_inflation_6.label = 'I6'

        with open(workdir+'/da_exp_I7.json', 'r') as fp:
            da_settings_inflation_7 = json.load(fp)
        exp_inflation_7 = load_or_run(da_settings_inflation_7)

        with open(workdir+'/da_exp_I8.json', 'r') as fp:
            da_settings_inflation_8 = json.load(fp)
        exp_inflation_8 = load_or_run(da_settings_inflation_8)

        for exp,settings,label in zip([exp_inflation_5,exp_inflation_6,exp_inflation_7,exp_inflation_8],\
                                      [da_settings_inflation_5,da_settings_inflation_6,da_settings_inflation_7,da_settings_inflation_8],
                                      [r"I$_5$",r"I$_6$",r"I$_7$",r"I$_8$"]):
            print(exp.inflation_rtps_alpha)

            plot_overview(exp,label,'exp_%s.png'%exp.label)

    if opt08:

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_O1 = json.load(fp)
        exp_O1 = load_or_run(da_settings_O1)
        exp_O1.label = 'O1'

        with open(workdir+'/da_exp_O2.json', 'r') as fp:
            da_settings_O2 = json.load(fp)
        exp_O2 = load_or_run(da_settings_O2)

        # Plots
        for exp,settings,label in zip([exp_O1,exp_O2],\
                                [da_settings_O1,da_settings_O2],\
                                [r"O$_1$",r"O$_2$"]):
            print('Random obs order? %s'%exp.randomize_obs)

            plot_overview(exp,label,'exp_%s.png'%exp.label)

        # Difference between two experiments
        nobs = exp_O1.nobs
        maxtime = exp_O1.nr.maxtime
        ntimes = exp_O1.da.ncycles
        me1 = np.zeros(exp_O1.da.model_equivalents.shape)+np.nan
        me2 = np.zeros(exp_O2.da.model_equivalents.shape)+np.nan

        # To get the plot right, we need to re-sort the
        # model equivalents and observation coordinates
        zobs1 = exp_O1.da.obs_coordinates
        zobs2 = exp_O2.da.obs_coordinates
        for k in range(ntimes):
            if is_sorted(zobs1[k,:]):
                indices = np.arange(nobs)
            else:
                indices = np.argsort(zobs1[k,:])
            zobs1[k,:] = zobs1[k,indices]
            me1[k,:,:] = exp_O1.da.model_equivalents[k,indices,:]
        for k in range(ntimes):
            if is_sorted(zobs2[k,:]):
                indices = np.arange(nobs)
            else:
                indices = np.argsort(zobs2[k,:])
            zobs2[k,:] = zobs2[k,indices]
            me2[k,:,:] = exp_O2.da.model_equivalents[k,indices,:]

        times = exp_O1.da.time
        times = times[:,None]+np.zeros((ntimes,nobs))
        delta_theta = me1.mean(axis=2) - me2.mean(axis=2)

        fig, ax = p.subplots(1,1,constrained_layout=True)
        fig.set_size_inches(4,3)
        c0 = ax.pcolormesh(times/3600.,zobs1,delta_theta,cmap='RdBu_r',norm=colors.CenteredNorm())
        ax.set_xlim([0,maxtime/3600])
        ax.set_xticks(np.arange((maxtime+1)/3600))
        ax.set_xlabel('Time (h)')
        ax.set_ylabel('Height (m)')
        ax.set_title(r'$\overline{y}_b$(O2)$ - \overline{y}_b$(O1) (K)')
        p.colorbar(c0,orientation='horizontal')
        fig.savefig('impact_of_random_obs.png',format='png',dpi=300)
        p.close(fig)

    if opt09:

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_A1 = json.load(fp)
        exp_A1 = load_or_run(da_settings_A1)

        plot_B_matrix(exp_A1,timeid=71)

    if opt10:

        with open(workdir+'/da_exp_B3.json', 'r') as fp:
            da_settings_B3 = json.load(fp)
        exp_B3 = load_or_run(da_settings_B3)

        with open(workdir+'/da_exp_D1.json', 'r') as fp:
            da_settings_D1 = json.load(fp)
        exp_D1 = load_or_run(da_settings_D1)

        with open(workdir+'/da_exp_D2.json', 'r') as fp:
            da_settings_D2 = json.load(fp)
        exp_D2 = load_or_run(da_settings_D2)

        # Plots
        plot_overview(exp_D1,r"D$_1$",'exp_D1.png')
        plot_overview(exp_D2,r"D$_2$",'exp_D2.png')
        plot_overview(exp_B3,r"B$_3$",'exp_B3.png')

        fig, [[ax1, ax2], [ax3, ax4]] = p.subplots(2,2,constrained_layout=True)
        fig.set_size_inches(6,4)
        plot_diagnostics(exp_D1,show='errors',axes=[ax1,ax2,ax3,ax4],label='D$_1$',linecolor='#1f77b4')
        plot_diagnostics(exp_D2,show='errors',axes=[ax1,ax2,ax3,ax4],label='D$_2$',linecolor='#9EC9E9')
        plot_diagnostics(exp_B3,show='errors',axes=[ax1,ax2,ax3,ax4],label='B$_3$',linecolor='orange')
        ax1.set_title('a) Analysis error')
        ax1.set_xlabel(r'RMSE$^a_\theta$')
        ax2.set_title('b) First-guess error')
        ax2.set_xlabel(r'RMSE$^b_\theta$')
        ax3.set_title('c) Error reduction')
        ax3.set_xlabel(r'RMSE$^b_\theta-$RMSE$^a_\theta$')
        ax4.set_title('d) Spread-error consistency')
        ax4.set_xlabel(r'$\sigma^b_\theta$/RMSE$^b_\theta$')
        ax1.set_ylabel('height (m)')
        ax3.set_ylabel('height (m)')
        ax1.legend(frameon=False,loc=1)
        #ax1.set_xlim([0,0.2])
        #ax2.set_xlim([0,0.2])
        #ax3.set_xlim([0,0.02])
        #ax4.set_xlim([0,10])
        fig.savefig('profiles_errors.png',format='png',dpi=300)
        p.close(fig)

        fig, [[ax1, ax2], [ax3, ax4]] = p.subplots(2,2,constrained_layout=True)
        fig.set_size_inches(6,4)
        plot_diagnostics(exp_D1,show='deviations',axes=[ax1,ax2,ax3,ax4],label='D$_1$',linecolor=u'#1f77b4')
        plot_diagnostics(exp_D2,show='deviations',axes=[ax1,ax2,ax3,ax4],label='D$_2$',linecolor=u'#9EC9E9')
        plot_diagnostics(exp_B3,show='deviations',axes=[ax1,ax2,ax3,ax4],label='B$_3$',linecolor='orange')
        ax1.set_title('a) Analysis error')
        ax1.set_xlabel(r'RMSD$^a_\theta$')
        ax2.set_title('b) First-guess error')
        ax2.set_xlabel(r'RMSD$^b_\theta$')
        ax3.set_title('c) Error reduction')
        ax3.set_xlabel(r'RMSD$^b_\theta-$RMSD$^a_\theta$')
        ax4.set_title('d) Spread-error consistency')
        ax4.set_xlabel(r'$\sigma^b_\theta$/RMSD$^b_\theta$')
        ax1.set_ylabel('height (m)')
        ax3.set_ylabel('height (m)')
        ax1.legend(frameon=False,loc=1)
        #ax1.set_xlim([0,0.2])
        #ax2.set_xlim([0,0.2])
        #ax3.set_xlim([0,0.02])
        #ax4.set_xlim([0,10])
        fig.savefig('profiles_deviations.png',format='png',dpi=300)
        p.close(fig)

    if opt11:

        with open(workdir+'/da_exp_I9.json', 'r') as fp:
            da_settings_I9 = json.load(fp)
        exp_I9 = load_or_run(da_settings_I9)

        # Plots
        plot_overview(exp_I9,r"I$_9$",'exp_I9.png')

    if opt12:

        with open(workdir+'/da_exp_A1.json', 'r') as fp:
            da_settings_A1 = json.load(fp)
        exp_A = load_or_run(da_settings_A1)

        with open(workdir+'/da_exp_D2.json', 'r') as fp:
            da_settings_D2 = json.load(fp)
        exp_D = load_or_run(da_settings_D2)

        fig, [[ax0, ax1],[ax2,ax3]] = p.subplots(2,2,constrained_layout=True)
        fig.set_size_inches(6,6)
        #
        [ax1,ax0],c0,c1 = plot_CBL_identifiability(exp_A,da_settings_A1['obs_error_sdev_assimilate'],ax=[ax1,ax0])
        ax0.set_title(r'a) Exp. A$_1$, $\delta\overline{y}$ (K)')
        ax0.set_xlabel('Time (h)')
        ax0.set_ylabel('Height (m)')
        ax1.set_title(r'b) Exp. A$_1$, $\sigma_{p\prime\prime}/\sigma_{y^b}$ (K$^{-1}$)')
        ax1.set_xlabel('Time (h)')
        ax1.set_ylabel('Height (m)')
        maxtime = exp_A.trun
        ax0.set_xlim([0,maxtime/3600])
        ax1.set_xlim([0,maxtime/3600])
        ax0.set_xticks(np.arange((maxtime+1)/3600))
        ax1.set_xticks(np.arange((maxtime+1)/3600))
        [ax3,ax2],c2,c3 = plot_CBL_identifiability(exp_D,da_settings_D2['obs_error_sdev_assimilate'],ax=[ax3,ax2])
        ax2.set_title(r'c) Exp. D$_2$, $\delta\overline{y}$ (K)')
        ax2.set_xlabel('Time (h)')
        ax2.set_ylabel('Height (m)')
        ax3.set_title(r'd) Exp. D$_2$, $\sigma_{p\prime\prime}/\sigma_{y^b}$ (K$^{-1}$)')
        ax3.set_xlabel('Time (h)')
        ax3.set_ylabel('Height (m)')
        maxtime = exp_D.trun
        ax2.set_xlim([0,maxtime/3600])
        ax3.set_xlim([0,maxtime/3600])
        ax2.set_xticks(np.arange((maxtime+1)/3600))
        ax3.set_xticks(np.arange((maxtime+1)/3600))
        p.colorbar(c0,orientation='horizontal')
        p.colorbar(c1,orientation='horizontal')
        p.colorbar(c2,orientation='horizontal')
        p.colorbar(c3,orientation='horizontal')
        #
        fig.savefig('fig08_old.png',format='png',dpi=300)
        p.close(fig)
    
    if check_reproducibility:

        for i in ["exp_A1.pickle",
                  "exp_A2.pickle",
                  "exp_B1.pickle",
                  "exp_B2.pickle",
                  "exp_B3.pickle",
                  "exp_B4.pickle",
                  "exp_C1.pickle",
                  "exp_C2.pickle",
                  "exp_D1.pickle",
                  "exp_D2.pickle"]:
            experiment = pickle.load(open(workdir+i, "rb"))
            print(i)
            print("OBS       :",experiment.da.observations[:,0])
            print("PARAMETERS:",experiment.da.initial_perturbed_parameters[0,:5])
            print("STATE     :",experiment.da.initial_state[0,0,:5])